/*
 * Defines the languages codes and abbreviations according to ISO 639-[12].
 * This is used in ld-address.c (address_finish).
 *
 * Format is: ("English name", 639-1-code, 639-2/T-code, 639-2/B-code)
 * If you find something missing or wrong, please go to the URL
 * http://www.gnu.org/software/libc/bugs.html and follow
 * instructions there to file a bug report.
 */

DEFINE_LANGUAGE_CODE ("Abkhazian", ab, abk, abk)
DEFINE_LANGUAGE_CODE3 ("Achinese", ace, ace)
DEFINE_LANGUAGE_CODE3 ("Acoli", ach, ach)
DEFINE_LANGUAGE_CODE3 ("Adangme", ada, ada)
DEFINE_LANGUAGE_CODE3 ("Adyghe; Adygei", ady, ady)
DEFINE_LANGUAGE_CODE ("Afar", aa, aar, aar)
DEFINE_LANGUAGE_CODE3 ("Afrihili", afh, afh)
DEFINE_LANGUAGE_CODE ("Afrikaans", af, afr, afr)
DEFINE_LANGUAGE_CODE3 ("Afro-Asiatic (Other)", afa, afa)
DEFINE_LANGUAGE_CODE3 ("Ainu", ain, ain)
DEFINE_LANGUAGE_CODE ("Akan", ak, aka, aka)
DEFINE_LANGUAGE_CODE3 ("Akkadian", akk, akk)
DEFINE_LANGUAGE_CODE ("Albanian", sq, sqi, alb)
DEFINE_LANGUAGE_CODE3 ("Aleut", ale, ale)
DEFINE_LANGUAGE_CODE3 ("Algonquian languages", alg, alg)
DEFINE_LANGUAGE_CODE3 ("Southern Altai", alt, alt)
DEFINE_LANGUAGE_CODE3 ("Altaic (Other)", tut, tut)
DEFINE_LANGUAGE_CODE ("Amharic", am, amh, amh)
DEFINE_LANGUAGE_CODE3 ("Angika", anp, anp)
DEFINE_LANGUAGE_CODE3 ("Apache languages", apa, apa)
DEFINE_LANGUAGE_CODE ("Arabic", ar, ara, ara)
DEFINE_LANGUAGE_CODE ("Aragonese", an, arg, arg)
DEFINE_LANGUAGE_CODE3 ("Aramaic", arc, arc)
DEFINE_LANGUAGE_CODE3 ("Arapaho", arp, arp)
DEFINE_LANGUAGE_CODE3 ("Araucanian", arn, arn)
DEFINE_LANGUAGE_CODE3 ("Arawak", arw, arw)
DEFINE_LANGUAGE_CODE ("Armenian", hy, hye, arm)
DEFINE_LANGUAGE_CODE3 ("Aromanian; Arumanian; Macedo-Romanian", rup, rup)
DEFINE_LANGUAGE_CODE3 ("Artificial (Other)", art, art)
DEFINE_LANGUAGE_CODE ("Assamese", as, asm, asm)
DEFINE_LANGUAGE_CODE3 ("Asturian; Bable", ast, ast)
DEFINE_LANGUAGE_CODE3 ("Athapascan languages", ath, ath)
DEFINE_LANGUAGE_CODE3 ("Australian languages", aus, aus)
DEFINE_LANGUAGE_CODE3 ("Austronesian (Other)", map, map)
DEFINE_LANGUAGE_CODE ("Avaric", av, ava, ava)
DEFINE_LANGUAGE_CODE ("Avestan", ae, ave, ave)
DEFINE_LANGUAGE_CODE3 ("Awadhi", awa, awa)
DEFINE_LANGUAGE_CODE ("Aymara", ay, aym, aym)
DEFINE_LANGUAGE_CODE ("Azerbaijani", az, aze, aze)
DEFINE_LANGUAGE_CODE3 ("Balinese", ban, ban)
DEFINE_LANGUAGE_CODE3 ("Baltic (Other)", bat, bat)
DEFINE_LANGUAGE_CODE3 ("Baluchi", bal, bal)
DEFINE_LANGUAGE_CODE ("Bambara", bm, bam, bam)
DEFINE_LANGUAGE_CODE3 ("Bamileke languages", bai, bai)
DEFINE_LANGUAGE_CODE3 ("Banda", bad, bad)
DEFINE_LANGUAGE_CODE3 ("Bantu (Other)", bnt, bnt)
DEFINE_LANGUAGE_CODE3 ("Basa", bas, bas)
DEFINE_LANGUAGE_CODE ("Bashkir", ba, bak, bak)
DEFINE_LANGUAGE_CODE ("Basque", eu, eus, baq)
DEFINE_LANGUAGE_CODE3 ("Batak (Indonesia)", btk, btk)
DEFINE_LANGUAGE_CODE3 ("Beja", bej, bej)
DEFINE_LANGUAGE_CODE ("Belarusian", be, bel, bel)
DEFINE_LANGUAGE_CODE3 ("Bemba", bem, bem)
DEFINE_LANGUAGE_CODE ("Bengali", bn, ben, ben)
DEFINE_LANGUAGE_CODE3 ("Berber (Other)", ber, ber)
DEFINE_LANGUAGE_CODE3 ("Bhojpuri", bho, bho)
DEFINE_LANGUAGE_CODE ("Bihari", bh, bih, bih)
DEFINE_LANGUAGE_CODE3 ("Bikol", bik, bik)
DEFINE_LANGUAGE_CODE3 ("Bini", bin, bin)
DEFINE_LANGUAGE_CODE ("Bislama", bi, bis, bis)
DEFINE_LANGUAGE_CODE3 ("Blin; Bilin", byn, byn)
DEFINE_LANGUAGE_CODE ("Bosnian", bs, bos, bos)
DEFINE_LANGUAGE_CODE3 ("Braj", bra, bra)
DEFINE_LANGUAGE_CODE ("Breton", br, bre, bre)
DEFINE_LANGUAGE_CODE3 ("Buginese", bug, bug)
DEFINE_LANGUAGE_CODE ("Bulgarian", bg, bul, bul)
DEFINE_LANGUAGE_CODE3 ("Buriat", bua, bua)
DEFINE_LANGUAGE_CODE ("Burmese", my, mya, bur)
DEFINE_LANGUAGE_CODE3 ("Caddo", cad, cad)
DEFINE_LANGUAGE_CODE3 ("Carib", car, car)
DEFINE_LANGUAGE_CODE ("Catalan; Valencian", ca, cat, cat)
DEFINE_LANGUAGE_CODE3 ("Caucasian (Other)", cau, cau)
DEFINE_LANGUAGE_CODE3 ("Cebuano", ceb, ceb)
DEFINE_LANGUAGE_CODE3 ("Celtic (Other)", cel, cel)
DEFINE_LANGUAGE_CODE3 ("Central American Indian (Other)", cai, cai)
DEFINE_LANGUAGE_CODE3 ("Chagatai", chg, chg)
DEFINE_LANGUAGE_CODE3 ("Chamic languages", cmc, cmc)
DEFINE_LANGUAGE_CODE ("Chamorro", ch, cha, cha)
DEFINE_LANGUAGE_CODE ("Chechen", ce, che, che)
DEFINE_LANGUAGE_CODE3 ("Cherokee", chr, chr)
DEFINE_LANGUAGE_CODE3 ("Cheyenne", chy, chy)
DEFINE_LANGUAGE_CODE3 ("Chibcha", chb, chb)
DEFINE_LANGUAGE_CODE ("Chichewa; Chewa; Nyanja", ny, nya, nya)
DEFINE_LANGUAGE_CODE ("Chinese", zh, zho, chi)
DEFINE_LANGUAGE_CODE3 ("Chinook jargon", chn, chn)
DEFINE_LANGUAGE_CODE3 ("Chipewyan", chp, chp)
DEFINE_LANGUAGE_CODE3 ("Choctaw", cho, cho)
DEFINE_LANGUAGE_CODE ("Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic", cu, chu, chu)
DEFINE_LANGUAGE_CODE3 ("Chuukese", chk, chk)
DEFINE_LANGUAGE_CODE ("Chuvash", cv, chv, chv)
DEFINE_LANGUAGE_CODE3 ("Classical Newari; Old Newari; Classical Nepal Bhasa", nwc, nwc)
DEFINE_LANGUAGE_CODE3 ("Coptic", cop, cop)
DEFINE_LANGUAGE_CODE ("Cornish", kw, cor, cor)
DEFINE_LANGUAGE_CODE ("Corsican", co, cos, cos)
DEFINE_LANGUAGE_CODE3 ("Creek", mus, mus)
DEFINE_LANGUAGE_CODE ("Cree", cr, cre, cre)
DEFINE_LANGUAGE_CODE3 ("Creoles and pidgins (Other)", crp, crp)
DEFINE_LANGUAGE_CODE3 ("Creoles and pidgins, English based (Other)", cpe, cpe)
DEFINE_LANGUAGE_CODE3 ("Creoles and pidgins, French-based (Other)", cpf, cpf)
DEFINE_LANGUAGE_CODE3 ("Creoles and pidgins, Portuguese-based (Other)", cpp, cpp)
DEFINE_LANGUAGE_CODE3 ("Crimean Tatar; Crimean Turkish", crh, crh)
DEFINE_LANGUAGE_CODE ("Croatian", hr, hrv, scr)
DEFINE_LANGUAGE_CODE3 ("Cushitic (Other)", cus, cus)
DEFINE_LANGUAGE_CODE ("Czech", cs, ces, cze)
DEFINE_LANGUAGE_CODE3 ("Dakota", dak, dak)
DEFINE_LANGUAGE_CODE ("Danish", da, dan, dan)
DEFINE_LANGUAGE_CODE3 ("Dargwa", dar, dar)
DEFINE_LANGUAGE_CODE3 ("Dayak", day, day)
DEFINE_LANGUAGE_CODE3 ("Delaware", del, del)
DEFINE_LANGUAGE_CODE3 ("Dinka", din, din)
DEFINE_LANGUAGE_CODE ("Divehi", dv, div, div)
DEFINE_LANGUAGE_CODE3 ("Dogrib", dgr, dgr)
DEFINE_LANGUAGE_CODE3 ("Dogri", doi, doi)
DEFINE_LANGUAGE_CODE3 ("Dravidian (Other)", dra, dra)
DEFINE_LANGUAGE_CODE3 ("Duala", dua, dua)
DEFINE_LANGUAGE_CODE3 ("Dutch, Middle (ca.1050-1350)", dum, dum)
DEFINE_LANGUAGE_CODE ("Dutch; Flemish", nl, nld, dut)
DEFINE_LANGUAGE_CODE3 ("Dyula", dyu, dyu)
DEFINE_LANGUAGE_CODE ("Dzongkha", dz, dzo, dzo)
DEFINE_LANGUAGE_CODE3 ("Efik", efi, efi)
DEFINE_LANGUAGE_CODE3 ("Egyptian (Ancient)", egy, egy)
DEFINE_LANGUAGE_CODE3 ("Ekajuk", eka, eka)
DEFINE_LANGUAGE_CODE3 ("Elamite", elx, elx)
DEFINE_LANGUAGE_CODE3 ("English, Middle (1100-1500)", enm, enm)
DEFINE_LANGUAGE_CODE3 ("English, Old (ca.450-1100)", ang, ang)
DEFINE_LANGUAGE_CODE ("English", en, eng, eng)
DEFINE_LANGUAGE_CODE3 ("Erzya", myv, myv)
DEFINE_LANGUAGE_CODE ("Esperanto", eo, epo, epo)
DEFINE_LANGUAGE_CODE ("Estonian", et, est, est)
DEFINE_LANGUAGE_CODE ("Ewe", ee, ewe, ewe)
DEFINE_LANGUAGE_CODE3 ("Ewondo", ewo, ewo)
DEFINE_LANGUAGE_CODE3 ("Fang", fan, fan)
DEFINE_LANGUAGE_CODE3 ("Fanti", fat, fat)
DEFINE_LANGUAGE_CODE ("Faroese", fo, fao, fao)
DEFINE_LANGUAGE_CODE ("Fijian", fj, fij, fij)
DEFINE_LANGUAGE_CODE3 ("Filipino; Pilipino", fil, fil)
DEFINE_LANGUAGE_CODE ("Finnish", fi, fin, fin)
DEFINE_LANGUAGE_CODE3 ("Finno-Ugrian (Other)", fiu, fiu)
DEFINE_LANGUAGE_CODE3 ("Fon", fon, fon)
DEFINE_LANGUAGE_CODE3 ("French, Middle (ca.1400-1800)", frm, frm)
DEFINE_LANGUAGE_CODE3 ("French, Old (842-ca.1400)", fro, fro)
DEFINE_LANGUAGE_CODE ("French", fr, fra, fre)
DEFINE_LANGUAGE_CODE3 ("Northern Frisian", frr, frr)
DEFINE_LANGUAGE_CODE3 ("Eastern Frisian", frs, frs)
DEFINE_LANGUAGE_CODE ("Western Frisian", fy, fry, fry)
DEFINE_LANGUAGE_CODE3 ("Friulian", fur, fur)
DEFINE_LANGUAGE_CODE ("Fulah", ff, ful, ful)
DEFINE_LANGUAGE_CODE ("Gaelic; Scottish Gaelic", gd, gla, gla)
DEFINE_LANGUAGE_CODE ("Gallegan", gl, glg, glg)
DEFINE_LANGUAGE_CODE ("Ganda", lg, lug, lug)
DEFINE_LANGUAGE_CODE3 ("Gayo", gay, gay)
DEFINE_LANGUAGE_CODE3 ("Ga", gaa, gaa)
DEFINE_LANGUAGE_CODE3 ("Gbaya", gba, gba)
DEFINE_LANGUAGE_CODE3 ("Geez", gez, gez)
DEFINE_LANGUAGE_CODE ("Georgian", ka, kat, geo)
DEFINE_LANGUAGE_CODE3 ("German, Middle High (ca.1050-1500)", gmh, gmh)
DEFINE_LANGUAGE_CODE3 ("German, Old High (ca.750-1050)", goh, goh)
DEFINE_LANGUAGE_CODE3 ("Germanic (Other)", gem, gem)
DEFINE_LANGUAGE_CODE ("German", de, deu, ger)
DEFINE_LANGUAGE_CODE3 ("Swiss German; Alemannic", gsw, gsw)
DEFINE_LANGUAGE_CODE3 ("Gilbertese", gil, gil)
DEFINE_LANGUAGE_CODE3 ("Gondi", gon, gon)
DEFINE_LANGUAGE_CODE3 ("Gorontalo", gor, gor)
DEFINE_LANGUAGE_CODE3 ("Gothic", got, got)
DEFINE_LANGUAGE_CODE3 ("Grebo", grb, grb)
DEFINE_LANGUAGE_CODE3 ("Greek, Ancient (to 1453)", grc, grc)
DEFINE_LANGUAGE_CODE ("Greek, Modern (1453-)", el, ell, gre)
DEFINE_LANGUAGE_CODE ("Guarani", gn, grn, grn)
DEFINE_LANGUAGE_CODE ("Gujarati", gu, guj, guj)
DEFINE_LANGUAGE_CODE3 ("Gwich´in", gwi, gwi)
DEFINE_LANGUAGE_CODE3 ("Haida", hai, hai)
DEFINE_LANGUAGE_CODE ("Haitian; Haitian Creole", ht, hat, hat)
DEFINE_LANGUAGE_CODE ("Hausa", ha, hau, hau)
DEFINE_LANGUAGE_CODE3 ("Hawaiian", haw, haw)
DEFINE_LANGUAGE_CODE ("Hebrew", he, heb, heb)
DEFINE_LANGUAGE_CODE ("Herero", hz, her, her)
DEFINE_LANGUAGE_CODE3 ("Hiligaynon", hil, hil)
DEFINE_LANGUAGE_CODE3 ("Himachali", him, him)
DEFINE_LANGUAGE_CODE ("Hindi", hi, hin, hin)
DEFINE_LANGUAGE_CODE ("Hiri Motu", ho, hmo, hmo)
DEFINE_LANGUAGE_CODE3 ("Hittite", hit, hit)
DEFINE_LANGUAGE_CODE3 ("Hmong", hmn, hmn)
DEFINE_LANGUAGE_CODE ("Hungarian", hu, hun, hun)
DEFINE_LANGUAGE_CODE3 ("Hupa", hup, hup)
DEFINE_LANGUAGE_CODE3 ("Iban", iba, iba)
DEFINE_LANGUAGE_CODE ("Icelandic", is, isl, ice)
DEFINE_LANGUAGE_CODE ("Ido", io, ido, ido)
DEFINE_LANGUAGE_CODE ("Igbo", ig, ibo, ibo)
DEFINE_LANGUAGE_CODE3 ("Ijo", ijo, ijo)
DEFINE_LANGUAGE_CODE3 ("Iloko", ilo, ilo)
DEFINE_LANGUAGE_CODE3 ("Inari Sami", smn, smn)
DEFINE_LANGUAGE_CODE3 ("Indic (Other)", inc, inc)
DEFINE_LANGUAGE_CODE3 ("Indo-European (Other)", ine, ine)
DEFINE_LANGUAGE_CODE ("Indonesian", id, ind, ind)
DEFINE_LANGUAGE_CODE3 ("Ingush", inh, inh)
DEFINE_LANGUAGE_CODE ("Interlingua (International Auxiliary Language Association)", ia, ina, ina)
DEFINE_LANGUAGE_CODE ("Interlingue", ie, ile, ile)
DEFINE_LANGUAGE_CODE ("Inuktitut", iu, iku, iku)
DEFINE_LANGUAGE_CODE ("Inupiaq", ik, ipk, ipk)
DEFINE_LANGUAGE_CODE3 ("Iranian (Other)", ira, ira)
DEFINE_LANGUAGE_CODE3 ("Irish, Middle (900-1200)", mga, mga)
DEFINE_LANGUAGE_CODE3 ("Irish, Old (to 900)", sga, sga)
DEFINE_LANGUAGE_CODE ("Irish", ga, gle, gle)
DEFINE_LANGUAGE_CODE3 ("Iroquoian languages", iro, iro)
DEFINE_LANGUAGE_CODE ("Italian", it, ita, ita)
DEFINE_LANGUAGE_CODE ("Japanese", ja, jpn, jpn)
DEFINE_LANGUAGE_CODE ("Javanese", jv, jav, jav)
DEFINE_LANGUAGE_CODE3 ("Judeo-Arabic", jrb, jrb)
DEFINE_LANGUAGE_CODE3 ("Judeo-Persian", jpr, jpr)
DEFINE_LANGUAGE_CODE3 ("Kabardian", kbd, kbd)
DEFINE_LANGUAGE_CODE3 ("Kabyle", kab, kab)
DEFINE_LANGUAGE_CODE3 ("Kachin", kac, kac)
DEFINE_LANGUAGE_CODE ("Kalaallisut; Greenlandic", kl, kal, kal)
DEFINE_LANGUAGE_CODE3 ("Kalmyk", xal, xal)
DEFINE_LANGUAGE_CODE3 ("Kamba", kam, kam)
DEFINE_LANGUAGE_CODE ("Kannada", kn, kan, kan)
DEFINE_LANGUAGE_CODE ("Kanuri", kr, kau, kau)
DEFINE_LANGUAGE_CODE3 ("Kara-Kalpak", kaa, kaa)
DEFINE_LANGUAGE_CODE3 ("Karachay-Balkar", krc, krc)
DEFINE_LANGUAGE_CODE3 ("Karelian", krl, krl)
DEFINE_LANGUAGE_CODE3 ("Karen", kar, kar)
DEFINE_LANGUAGE_CODE ("Kashmiri", ks, kas, kas)
DEFINE_LANGUAGE_CODE3 ("Kashubian", csb, csb)
DEFINE_LANGUAGE_CODE3 ("Kawi", kaw, kaw)
DEFINE_LANGUAGE_CODE ("Kazakh", kk, kaz, kaz)
DEFINE_LANGUAGE_CODE3 ("Khasi", kha, kha)
DEFINE_LANGUAGE_CODE ("Khmer", km, khm, khm)
DEFINE_LANGUAGE_CODE3 ("Khoisan (Other)", khi, khi)
DEFINE_LANGUAGE_CODE3 ("Khotanese", kho, kho)
DEFINE_LANGUAGE_CODE ("Kikuyu; Gikuyu", ki, kik, kik)
DEFINE_LANGUAGE_CODE3 ("Kimbundu", kmb, kmb)
DEFINE_LANGUAGE_CODE ("Kinyarwanda", rw, kin, kin)
DEFINE_LANGUAGE_CODE ("Kirghiz", ky, kir, kir)
DEFINE_LANGUAGE_CODE3 ("Klingon; tlhIngan-Hol", tlh, tlh)
DEFINE_LANGUAGE_CODE ("Komi", kv, kom, kom)
DEFINE_LANGUAGE_CODE ("Kongo", kg, kon, kon)
DEFINE_LANGUAGE_CODE3 ("Konkani", kok, kok)
DEFINE_LANGUAGE_CODE ("Korean", ko, kor, kor)
DEFINE_LANGUAGE_CODE3 ("Kosraean", kos, kos)
DEFINE_LANGUAGE_CODE3 ("Kpelle", kpe, kpe)
DEFINE_LANGUAGE_CODE3 ("Kru", kro, kro)
DEFINE_LANGUAGE_CODE ("Kuanyama; Kwanyama", kj, kua, kua)
DEFINE_LANGUAGE_CODE3 ("Kumyk", kum, kum)
DEFINE_LANGUAGE_CODE ("Kurdish", ku, kur, kur)
DEFINE_LANGUAGE_CODE3 ("Kurukh", kru, kru)
DEFINE_LANGUAGE_CODE3 ("Kutenai", kut, kut)
DEFINE_LANGUAGE_CODE3 ("Ladino", lad, lad)
DEFINE_LANGUAGE_CODE3 ("Lahnda", lah, lah)
DEFINE_LANGUAGE_CODE3 ("Lamba", lam, lam)
DEFINE_LANGUAGE_CODE ("Lao", lo, lao, lao)
DEFINE_LANGUAGE_CODE ("Latin", la, lat, lat)
DEFINE_LANGUAGE_CODE ("Latvian", lv, lav, lav)
DEFINE_LANGUAGE_CODE3 ("Lezghian", lez, lez)
DEFINE_LANGUAGE_CODE ("Limburgan; Limburger; Limburgish", li, lim, lim)
DEFINE_LANGUAGE_CODE ("Lingala", ln, lin, lin)
DEFINE_LANGUAGE_CODE ("Lithuanian", lt, lit, lit)
DEFINE_LANGUAGE_CODE3 ("Lojban", jbo, jbo)
DEFINE_LANGUAGE_CODE3 ("Low German; Low Saxon; German, Low; Saxon, Low", nds, nds)
DEFINE_LANGUAGE_CODE3 ("Lower Sorbian", dsb, dsb)
DEFINE_LANGUAGE_CODE3 ("Lozi", loz, loz)
DEFINE_LANGUAGE_CODE ("Luba-Katanga", lu, lub, lub)
DEFINE_LANGUAGE_CODE3 ("Luba-Lulua", lua, lua)
DEFINE_LANGUAGE_CODE3 ("Luiseno", lui, lui)
DEFINE_LANGUAGE_CODE3 ("Lule Sami", smj, smj)
DEFINE_LANGUAGE_CODE3 ("Lunda", lun, lun)
DEFINE_LANGUAGE_CODE3 ("Luo (Kenya and Tanzania)", luo, luo)
DEFINE_LANGUAGE_CODE3 ("Lushai", lus, lus)
DEFINE_LANGUAGE_CODE ("Luxembourgish; Letzeburgesch", lb, ltz, ltz)
DEFINE_LANGUAGE_CODE ("Macedonian", mk, mkd, mac)
DEFINE_LANGUAGE_CODE3 ("Madurese", mad, mad)
DEFINE_LANGUAGE_CODE3 ("Magahi", mag, mag)
DEFINE_LANGUAGE_CODE3 ("Maithili", mai, mai)
DEFINE_LANGUAGE_CODE3 ("Makasar", mak, mak)
DEFINE_LANGUAGE_CODE ("Malagasy", mg, mlg, mlg)
DEFINE_LANGUAGE_CODE ("Malayalam", ml, mal, mal)
DEFINE_LANGUAGE_CODE ("Malay", ms, msa, may)
DEFINE_LANGUAGE_CODE ("Maltese", mt, mlt, mlt)
DEFINE_LANGUAGE_CODE3 ("Manchu", mnc, mnc)
DEFINE_LANGUAGE_CODE3 ("Mandar", mdr, mdr)
DEFINE_LANGUAGE_CODE3 ("Mandingo", man, man)
DEFINE_LANGUAGE_CODE3 ("Manipuri", mni, mni)
DEFINE_LANGUAGE_CODE3 ("Manobo languages", mno, mno)
DEFINE_LANGUAGE_CODE ("Manx", gv, glv, glv)
DEFINE_LANGUAGE_CODE ("Maori", mi, mri, mao)
DEFINE_LANGUAGE_CODE ("Marathi", mr, mar, mar)
DEFINE_LANGUAGE_CODE3 ("Mari", chm, chm)
DEFINE_LANGUAGE_CODE ("Marshallese", mh, mah, mah)
DEFINE_LANGUAGE_CODE3 ("Marwari", mwr, mwr)
DEFINE_LANGUAGE_CODE3 ("Masai", mas, mas)
DEFINE_LANGUAGE_CODE3 ("Mayan languages", myn, myn)
DEFINE_LANGUAGE_CODE3 ("Mende", men, men)
DEFINE_LANGUAGE_CODE3 ("Mi'kmaq; Micmac", mic, mic)
DEFINE_LANGUAGE_CODE3 ("Minangkabau", min, min)
DEFINE_LANGUAGE_CODE3 ("Mirandese", mwl, mwl)
DEFINE_LANGUAGE_CODE3 ("Miscellaneous languages", mis, mis)
DEFINE_LANGUAGE_CODE3 ("Mohawk", moh, moh)
DEFINE_LANGUAGE_CODE3 ("Moksha", mdf, mdf)
DEFINE_LANGUAGE_CODE ("Moldavian", mo, mol, mol)
DEFINE_LANGUAGE_CODE3 ("Mon-Khmer (Other)", mkh, mkh)
DEFINE_LANGUAGE_CODE ("Mongolian", mn, mon, mon)
DEFINE_LANGUAGE_CODE3 ("Mongo", lol, lol)
DEFINE_LANGUAGE_CODE3 ("Mossi", mos, mos)
DEFINE_LANGUAGE_CODE3 ("Multiple languages", mul, mul)
DEFINE_LANGUAGE_CODE3 ("Munda languages", mun, mun)
DEFINE_LANGUAGE_CODE3 ("Nahuatl", nah, nah)
DEFINE_LANGUAGE_CODE ("Nauru", na, nau, nau)
DEFINE_LANGUAGE_CODE ("Navajo; Navaho", nv, nav, nav)
DEFINE_LANGUAGE_CODE ("Ndebele, North; North Ndebele", nd, nde, nde)
DEFINE_LANGUAGE_CODE ("Ndebele, South; South Ndebele", nr, nbl, nbl)
DEFINE_LANGUAGE_CODE ("Ndonga", ng, ndo, ndo)
DEFINE_LANGUAGE_CODE3 ("Neapolitan", nap, nap)
DEFINE_LANGUAGE_CODE3 ("Nepal Bhasa; Newari", new, new)
DEFINE_LANGUAGE_CODE ("Nepali", ne, nep, nep)
DEFINE_LANGUAGE_CODE3 ("Nias", nia, nia)
DEFINE_LANGUAGE_CODE3 ("Niger-Kordofanian (Other)", nic, nic)
DEFINE_LANGUAGE_CODE3 ("Nilo-Saharan (Other)", ssa, ssa)
DEFINE_LANGUAGE_CODE3 ("Niuean", niu, niu)
DEFINE_LANGUAGE_CODE3 ("N'Ko", nqo, nqo)
DEFINE_LANGUAGE_CODE3 ("Nogai", nog, nog)
DEFINE_LANGUAGE_CODE3 ("Norse, Old", non, non)
DEFINE_LANGUAGE_CODE3 ("North American Indian", nai, nai)
DEFINE_LANGUAGE_CODE ("Northern Sami", se, sme, sme)
DEFINE_LANGUAGE_CODE3 ("Northern Sotho; Pedi; Sepedi", nso, nso)
DEFINE_LANGUAGE_CODE ("Norwegian Bokmċl", nb, nob, nob)
DEFINE_LANGUAGE_CODE ("Norwegian Nynorsk", nn, nno, nno)
DEFINE_LANGUAGE_CODE ("Norwegian", no, nor, nor)
DEFINE_LANGUAGE_CODE3 ("Nubian languages", nub, nub)
DEFINE_LANGUAGE_CODE3 ("Nyamwezi", nym, nym)
DEFINE_LANGUAGE_CODE3 ("Nyankole", nyn, nyn)
DEFINE_LANGUAGE_CODE3 ("Nyoro", nyo, nyo)
DEFINE_LANGUAGE_CODE3 ("Nzima", nzi, nzi)
DEFINE_LANGUAGE_CODE ("Occitan (post 1500); Provençal", oc, oci, oci)
DEFINE_LANGUAGE_CODE ("Ojibwa", oj, oji, oji)
DEFINE_LANGUAGE_CODE ("Oriya", or, ori, ori)
DEFINE_LANGUAGE_CODE ("Oromo", om, orm, orm)
DEFINE_LANGUAGE_CODE3 ("Osage", osa, osa)
DEFINE_LANGUAGE_CODE ("Ossetian; Ossetic", os, oss, oss)
DEFINE_LANGUAGE_CODE3 ("Otomian languages", oto, oto)
DEFINE_LANGUAGE_CODE3 ("Pahlavi", pal, pal)
DEFINE_LANGUAGE_CODE3 ("Palauan", pau, pau)
DEFINE_LANGUAGE_CODE ("Pali", pi, pli, pli)
DEFINE_LANGUAGE_CODE3 ("Pampanga", pam, pam)
DEFINE_LANGUAGE_CODE3 ("Pangasinan", pag, pag)
DEFINE_LANGUAGE_CODE ("Panjabi; Punjabi", pa, pan, pan)
DEFINE_LANGUAGE_CODE3 ("Papiamento", pap, pap)
DEFINE_LANGUAGE_CODE3 ("Papuan (Other)", paa, paa)
DEFINE_LANGUAGE_CODE3 ("Persian, Old (ca.600-400 B.C.)", peo, peo)
DEFINE_LANGUAGE_CODE ("Persian", fa, fas, per)
DEFINE_LANGUAGE_CODE3 ("Philippine (Other)", phi, phi)
DEFINE_LANGUAGE_CODE3 ("Phoenician", phn, phn)
DEFINE_LANGUAGE_CODE3 ("Pohnpeian", pon, pon)
DEFINE_LANGUAGE_CODE ("Polish", pl, pol, pol)
DEFINE_LANGUAGE_CODE ("Portuguese", pt, por, por)
DEFINE_LANGUAGE_CODE3 ("Prakrit languages", pra, pra)
DEFINE_LANGUAGE_CODE3 ("Provençal, Old (to 1500)", pro, pro)
DEFINE_LANGUAGE_CODE ("Pushto", ps, pus, pus)
DEFINE_LANGUAGE_CODE ("Quechua", qu, que, que)
DEFINE_LANGUAGE_CODE ("Raeto-Romance", rm, roh, roh)
DEFINE_LANGUAGE_CODE3 ("Rajasthani", raj, raj)
DEFINE_LANGUAGE_CODE3 ("Rapanui", rap, rap)
DEFINE_LANGUAGE_CODE3 ("Rarotongan", rar, rar)
DEFINE_LANGUAGE_CODE3 ("Romance (Other)", roa, roa)
DEFINE_LANGUAGE_CODE ("Romanian", ro, ron, rum)
DEFINE_LANGUAGE_CODE3 ("Romany", rom, rom)
DEFINE_LANGUAGE_CODE ("Rundi", rn, run, run)
DEFINE_LANGUAGE_CODE ("Russian", ru, rus, rus)
DEFINE_LANGUAGE_CODE3 ("Salishan languages", sal, sal)
DEFINE_LANGUAGE_CODE3 ("Samaritan Aramaic", sam, sam)
DEFINE_LANGUAGE_CODE3 ("Sami languages (Other)", smi, smi)
DEFINE_LANGUAGE_CODE ("Samoan", sm, smo, smo)
DEFINE_LANGUAGE_CODE3 ("Sandawe", sad, sad)
DEFINE_LANGUAGE_CODE ("Sango", sg, sag, sag)
DEFINE_LANGUAGE_CODE ("Sanskrit", sa, san, san)
DEFINE_LANGUAGE_CODE3 ("Santali", sat, sat)
DEFINE_LANGUAGE_CODE ("Sardinian", sc, srd, srd)
DEFINE_LANGUAGE_CODE3 ("Sasak", sas, sas)
DEFINE_LANGUAGE_CODE3 ("Scots", sco, sco)
DEFINE_LANGUAGE_CODE3 ("Selkup", sel, sel)
DEFINE_LANGUAGE_CODE3 ("Semitic (Other)", sem, sem)
DEFINE_LANGUAGE_CODE ("Serbian", sr, srp, scc)
DEFINE_LANGUAGE_CODE3 ("Serer", srr, srr)
DEFINE_LANGUAGE_CODE3 ("Shan", shn, shn)
DEFINE_LANGUAGE_CODE ("Shona", sn, sna, sna)
DEFINE_LANGUAGE_CODE ("Sichuan Yi", ii, iii, iii)
DEFINE_LANGUAGE_CODE3 ("Sicilian", scn, scn)
DEFINE_LANGUAGE_CODE3 ("Sidamo", sid, sid)
DEFINE_LANGUAGE_CODE3 ("Sign Languages", sgn, sgn)
DEFINE_LANGUAGE_CODE3 ("Siksika", bla, bla)
DEFINE_LANGUAGE_CODE ("Sindhi", sd, snd, snd)
DEFINE_LANGUAGE_CODE ("Sinhala; Sinhalese", si, sin, sin)
DEFINE_LANGUAGE_CODE3 ("Sino-Tibetan (Other)", sit, sit)
DEFINE_LANGUAGE_CODE3 ("Siouan languages", sio, sio)
DEFINE_LANGUAGE_CODE3 ("Skolt Sami", sms, sms)
DEFINE_LANGUAGE_CODE3 ("Slave (Athapascan)", den, den)
DEFINE_LANGUAGE_CODE3 ("Slavic (Other)", sla, sla)
DEFINE_LANGUAGE_CODE ("Slovak", sk, slk, slo)
DEFINE_LANGUAGE_CODE ("Slovenian", sl, slv, slv)
DEFINE_LANGUAGE_CODE3 ("Sogdian", sog, sog)
DEFINE_LANGUAGE_CODE ("Somali", so, som, som)
DEFINE_LANGUAGE_CODE3 ("Songhai", son, son)
DEFINE_LANGUAGE_CODE3 ("Soninke", snk, snk)
DEFINE_LANGUAGE_CODE3 ("Sorbian languages", wen, wen)
DEFINE_LANGUAGE_CODE ("Sotho, Southern", st, sot, sot)
DEFINE_LANGUAGE_CODE3 ("South American Indian (Other)", sai, sai)
DEFINE_LANGUAGE_CODE3 ("Southern Sami", sma, sma)
DEFINE_LANGUAGE_CODE ("Spanish; Castilian", es, spa, spa)
DEFINE_LANGUAGE_CODE3 ("Sranan Tongo", srn, srn)
DEFINE_LANGUAGE_CODE3 ("Sukuma", suk, suk)
DEFINE_LANGUAGE_CODE3 ("Sumerian", sux, sux)
DEFINE_LANGUAGE_CODE ("Sundanese", su, sun, sun)
DEFINE_LANGUAGE_CODE3 ("Susu", sus, sus)
DEFINE_LANGUAGE_CODE ("Swahili", sw, swa, swa)
DEFINE_LANGUAGE_CODE ("Swati", ss, ssw, ssw)
DEFINE_LANGUAGE_CODE ("Swedish", sv, swe, swe)
DEFINE_LANGUAGE_CODE3 ("Classical Syriac", syc, syc)
DEFINE_LANGUAGE_CODE3 ("Syriac", syr, syr)
DEFINE_LANGUAGE_CODE ("Tagalog", tl, tgl, tgl)
DEFINE_LANGUAGE_CODE ("Tahitian", ty, tah, tah)
DEFINE_LANGUAGE_CODE3 ("Tai (Other)", tai, tai)
DEFINE_LANGUAGE_CODE ("Tajik", tg, tgk, tgk)
DEFINE_LANGUAGE_CODE3 ("Tamashek", tmh, tmh)
DEFINE_LANGUAGE_CODE ("Tamil", ta, tam, tam)
DEFINE_LANGUAGE_CODE ("Tatar", tt, tat, tat)
DEFINE_LANGUAGE_CODE ("Telugu", te, tel, tel)
DEFINE_LANGUAGE_CODE3 ("Tereno", ter, ter)
DEFINE_LANGUAGE_CODE3 ("Tetum", tet, tet)
DEFINE_LANGUAGE_CODE ("Thai", th, tha, tha)
DEFINE_LANGUAGE_CODE ("Tibetan", bo, bod, tib)
DEFINE_LANGUAGE_CODE3 ("Tigre", tig, tig)
DEFINE_LANGUAGE_CODE ("Tigrinya", ti, tir, tir)
DEFINE_LANGUAGE_CODE3 ("Timne", tem, tem)
DEFINE_LANGUAGE_CODE3 ("Tiv", tiv, tiv)
DEFINE_LANGUAGE_CODE3 ("Tlingit", tli, tli)
DEFINE_LANGUAGE_CODE3 ("Tok Pisin", tpi, tpi)
DEFINE_LANGUAGE_CODE3 ("Tokelau", tkl, tkl)
DEFINE_LANGUAGE_CODE3 ("Tonga (Nyasa)", tog, tog)
DEFINE_LANGUAGE_CODE ("Tonga (Tonga Islands)", to, ton, ton)
DEFINE_LANGUAGE_CODE3 ("Tsimshian", tsi, tsi)
DEFINE_LANGUAGE_CODE ("Tsonga", ts, tso, tso)
DEFINE_LANGUAGE_CODE ("Tswana", tn, tsn, tsn)
DEFINE_LANGUAGE_CODE3 ("Tumbuka", tum, tum)
DEFINE_LANGUAGE_CODE3 ("Tupi languages", tup, tup)
DEFINE_LANGUAGE_CODE3 ("Turkish, Ottoman (1500-1928)", ota, ota)
DEFINE_LANGUAGE_CODE ("Turkish", tr, tur, tur)
DEFINE_LANGUAGE_CODE ("Turkmen", tk, tuk, tuk)
DEFINE_LANGUAGE_CODE3 ("Tuvalu", tvl, tvl)
DEFINE_LANGUAGE_CODE3 ("Tuvinian", tyv, tyv)
DEFINE_LANGUAGE_CODE ("Twi", tw, twi, twi)
DEFINE_LANGUAGE_CODE3 ("Udmurt", udm, udm)
DEFINE_LANGUAGE_CODE3 ("Ugaritic", uga, uga)
DEFINE_LANGUAGE_CODE ("Uighur; Uyghur", ug, uig, uig)
DEFINE_LANGUAGE_CODE ("Ukrainian", uk, ukr, ukr)
DEFINE_LANGUAGE_CODE3 ("Umbundu", umb, umb)
DEFINE_LANGUAGE_CODE3 ("Undetermined", und, und)
DEFINE_LANGUAGE_CODE3 ("Upper Sorbian", hsb, hsb)
DEFINE_LANGUAGE_CODE ("Urdu", ur, urd, urd)
DEFINE_LANGUAGE_CODE ("Uzbek", uz, uzb, uzb)
DEFINE_LANGUAGE_CODE3 ("Vai", vai, vai)
DEFINE_LANGUAGE_CODE ("Venda", ve, ven, ven)
DEFINE_LANGUAGE_CODE ("Vietnamese", vi, vie, vie)
DEFINE_LANGUAGE_CODE ("Volapük", vo, vol, vol)
DEFINE_LANGUAGE_CODE3 ("Votic", vot, vot)
DEFINE_LANGUAGE_CODE3 ("Wakashan languages", wak, wak)
DEFINE_LANGUAGE_CODE3 ("Walamo", wal, wal)
DEFINE_LANGUAGE_CODE ("Walloon", wa, wln, wln)
DEFINE_LANGUAGE_CODE3 ("Waray", war, war)
DEFINE_LANGUAGE_CODE3 ("Washo", was, was)
DEFINE_LANGUAGE_CODE ("Welsh", cy, cym, wel)
DEFINE_LANGUAGE_CODE ("Wolof", wo, wol, wol)
DEFINE_LANGUAGE_CODE ("Xhosa", xh, xho, xho)
DEFINE_LANGUAGE_CODE3 ("Yakut", sah, sah)
DEFINE_LANGUAGE_CODE3 ("Yao", yao, yao)
DEFINE_LANGUAGE_CODE3 ("Yapese", yap, yap)
DEFINE_LANGUAGE_CODE ("Yiddish", yi, yid, yid)
DEFINE_LANGUAGE_CODE ("Yoruba", yo, yor, yor)
DEFINE_LANGUAGE_CODE3 ("Yupik languages", ypk, ypk)
DEFINE_LANGUAGE_CODE3 ("Zande", znd, znd)
DEFINE_LANGUAGE_CODE3 ("Zapotec", zap, zap)
DEFINE_LANGUAGE_CODE3 ("Zenaga", zen, zen)
DEFINE_LANGUAGE_CODE ("Zhuang; Chuang", za, zha, zha)
DEFINE_LANGUAGE_CODE ("Zulu", zu, zul, zul)
DEFINE_LANGUAGE_CODE3 ("Zuni", zun, zun)
DEFINE_LANGUAGE_CODE3 ("Zaza; Dimili; Dimli; Kirdki; Kirmanjki; Zazaki", zza, zza)
